/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008-2011 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

/**
 * UGENE - Integrated Bioinformatics Tools.
 * Copyright (C) 2008-2011 UniPro <ugene@unipro.ru>
 * http://ugene.unipro.ru
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */

#ifndef _U2_WORKFLOW_ENV_H_
#define _U2_WORKFLOW_ENV_H_

#include <assert.h>
#include <U2Lang/Datatype.h>
#include "Descriptor.h"

namespace U2 {

class DataTypeRegistry;

namespace Workflow {

class DomainFactoryRegistry;
class ActorPrototypeRegistry;

/**
 * base class for workflow environment
 * has one instance
 * same as AppContext only for workflow
 * 
 * see WorkflowEnvImpl for realization
 */
class U2LANG_EXPORT WorkflowEnv {
public:
    static bool init(WorkflowEnv* instance);
    static void shutdown() { delete instance; instance = NULL;}
    
    static DataTypeRegistry* getDataTypeRegistry() {return getInstance()->data;}
    static ActorPrototypeRegistry* getProtoRegistry() {return getInstance()->proto;}
    static DomainFactoryRegistry* getDomainRegistry() {return getInstance()->domain;}
    static DataTypeValueFactoryRegistry* getDataTypeValueFactoryRegistry() { return getInstance()->dvfReg; }
    
protected:
    static WorkflowEnv* instance;
    static WorkflowEnv* getInstance() {assert(instance); return instance;}
    
public:
    virtual ~WorkflowEnv() {}
    
protected:
    virtual DataTypeRegistry* initDataRegistry() = 0;
    virtual ActorPrototypeRegistry* initProtoRegistry() = 0;
    virtual DomainFactoryRegistry* initDomainRegistry() = 0;
    virtual DataTypeValueFactoryRegistry* initDataTypeValueFactoryRegistry() = 0;
    
protected:
    DataTypeRegistry* data;
    ActorPrototypeRegistry* proto;
    DomainFactoryRegistry* domain;
    DataTypeValueFactoryRegistry* dvfReg;
    
}; // WorkflowEnv

} //namespace Workflow

} //namespace U2

#endif // _U2_WORKFLOW_ENV_H_
